@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2022	PgPool Global Development Group
+ * Copyright (c) 2003-2020	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -20,6 +20,9 @@
  */
 #include "pool.h"
 #include "pool_config.h"
+#include "version.h"
+#include "pool_config_variables.h"
+
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -35,17 +38,20 @@
 #include <errno.h>
 #include <string.h>
 #include <libgen.h>
+
 #include "utils/elog.h"
 #include "utils/palloc.h"
 #include "utils/memutils.h"
 #include "utils/pool_path.h"
+#include "utils/pool_signal.h"
+#include "utils/pool_ipc.h"
+#include "utils/ps_status.h"
+#include "utils/pool_ssl.h"
 
-#include "version.h"
 #include "auth/pool_passwd.h"
 #include "auth/pool_hba.h"
 #include "query_cache/pool_memqcache.h"
 #include "watchdog/wd_utils.h"
-#include "pool_config_variables.h"
 
 
 static bool get_pool_key_filename(char *poolKeyFile);
@@ -216,6 +222,8 @@ main(int argc, char **argv)
 	hba_file = make_absolute_path(hba_file_path, base_dir);
 
 	mypid = getpid();
+	SetProcessGlobalVaraibles(PT_MAIN);
+
 
 	pool_init_config();
 
@@ -473,6 +481,7 @@ daemonize(void)
 #endif
 
 	mypid = getpid();
+	SetProcessGlobalVaraibles(PT_MAIN);
 	write_pid_file();
 	if (chdir("/"))
 		ereport(WARNING,
@@ -522,31 +531,19 @@ stop_me(void)
 				(errmsg("could not read pid file")));
 	}
 
-	for (;;)
+	if (kill(pid, stop_sig) == -1)
 	{
-		int		cnt = 5;	/* sending signal retry interval */
+		ereport(FATAL,
+				(errmsg("could not stop process with pid: %d", pid),
+				 errdetail("%m")));
+	}
+	ereport(LOG,
+			(errmsg("stop request sent to pgpool. waiting for termination...")));
 
-		if (kill(pid, stop_sig) == -1)
-		{
-			ereport(FATAL,
-					(errmsg("could not stop process with pid: %d", pid),
-					 errdetail("%m")));
-		}
-		ereport(LOG,
-				(errmsg("stop request sent to pgpool (pid: %d). waiting for termination...", pid)));
-
-		while (kill(pid, 0) == 0)
-		{
-			fprintf(stderr, ".");
-			sleep(1);
-			cnt--;
-			/* If pgpool did not stop within 5 seconds, break the loop and try
-			 * to send the signal again */
-			if (cnt <= 0)
-				break;
-		}
-		if (cnt > 0)
-			break;
+	while (kill(pid, 0) == 0)
+	{
+		fprintf(stderr, ".");
+		sleep(1);
 	}
 	fprintf(stderr, "done.\n");
 	pid_file = get_pid_file_path();
